@@ -8,7 +8,7 @@ use Cwd 'getcwd';
 
 use 5.014;
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 =encoding utf8
 
@@ -99,6 +99,8 @@ sub run {
     my ($self, $command, %data) = @_;
     my $future = loop->new_future;
 
+    bless $future, 'App::Multigit::Future';
+
     $data{stdout} //= '';
 
     my $ignore_stdout = $App::Multigit::BEHAVIOUR{ignore_stdout};
@@ -114,13 +116,20 @@ sub run {
             ],
             on_finish => sub {
                 my (undef, $exitcode, $stdout, $stderr) = @_;
-                $future->done(
+                my %details = (
                     stdout => $ignore_stdout ? '' : $stdout,
                     stderr => $ignore_stderr ? '' : $stderr, 
                     exitcode => $exitcode, 
                     past_stdout => $ignore_stdout ? '' : $data{stdout},
                     past_stderr => $ignore_stderr ? '' : $data{stderr},
                 );
+
+                if ($exitcode == 0) {
+                    $future->done(%details);
+                }
+                else {
+                    $future->fail("Child process exited with nonzero exit status", %details);
+                }
             }
         );
     }
@@ -133,13 +142,22 @@ sub run {
             stdin => $data{stdout},
             on_finish => sub {
                 my (undef, $exitcode, $stdout, $stderr) = @_;
-                $future->done(
+                my %details = (
                     stdout => $ignore_stdout ? '' : $stdout,
                     stderr => $ignore_stderr ? '' : $stderr,
                     exitcode => $exitcode,
                     past_stdout => $ignore_stdout ? '' : $data{stdout},
                     past_stderr => $ignore_stderr ? '' : $data{stderr},
                 );
+
+                if ($exitcode == 0) {
+                    $future->done(%details);
+                }
+                else {
+                    $future->fail(
+                        "Child process exited with nonzero exit status",
+                        exit_nonzero => %details);
+                }
             }
         )
     }
